//
//  RingsView.swift
//  swoof
//
//  Created by Dillon DuPont on 12/6/18.
//  Copyright © 2018 Dillon DuPont. All rights reserved.
//

import UIKit

class RingsView: UIView {
    var shapes = ["ring.png", "square.png", "star.png", "triangle.png"];
    var circles : [UIImageView] = [];
    
    func setup() {
        for view in self.subviews {
            view.removeFromSuperview()
        }
        
        let startRadius = CGFloat(50.0);
        let finalRadius = CGFloat(600.0);
        for i in 0...4 {
            let circle = Circle(
                frame: CGRect(
                    x: (self.frame.width / 2) - startRadius,
                    y: (self.frame.height / 2) - startRadius,
                    width: startRadius * 2,
                    height: startRadius * 2),
                strokeColor: UIColor(white: 1.0, alpha: 0.1), fillColor: .clear);
            self.addSubview(circle);
            
            UIView.animate(withDuration: 40.0,
                           delay: (TimeInterval(CGFloat(i) * 8)),
                           options: [.repeat, .curveEaseInOut],
                           animations: { () -> Void in
                            circle.frame = CGRect(
                                x: (self.frame.width / 2) - finalRadius,
                                y: (self.frame.height / 2) - finalRadius,
                                width: finalRadius * 2,
                                height: finalRadius * 2);
            }, completion: { (finished) -> Void in
                
            })
            
            Timer.scheduledTimer(timeInterval: Double.random(in: 0.0 ... 8.0), target: self, selector: #selector(self.addCircle), userInfo: nil, repeats: false)
        }
    }
    
    @objc func addCircle() {
        let size = CGFloat.random(in: 20 ... 50);
        let x = CGFloat.random(in: 0 ... self.frame.width);
        let y = CGFloat.random(in: 0 ... self.frame.height);
        let minDist = CGFloat(120.0);
        
        var flag = sqrt(x * x + y * y) > minDist;
        
        for iv in circles {
            let xd = iv.center.x - x;
            let yd = iv.center.y - y;
            if(sqrt(xd * xd + yd * yd) < minDist) {
                flag = false;
            }
        }
        
        if(flag) {
            let imageView = UIImageView(frame: CGRect(x: x - (size / 2), y: y - (size / 2), width: size, height: size))
            imageView.image = UIImage(named: shapes.randomElement()!)?.withRenderingMode(.alwaysTemplate);
            imageView.transform = CGAffineTransform(rotationAngle: CGFloat(Float(arc4random()) / Float(UINT32_MAX)));
            self.addSubview(imageView)
            self.circles.append(imageView);
            
            imageView.alpha = 0;
            imageView.tintColor = UIColor(hue: CGFloat.random(in: 0 ... 1.0), saturation: 0.4, brightness: 1.0, alpha: CGFloat.random(in: 0.3 ... 1));
            //let speed = CGFloat.random(in: 0 ... 1);
            let velocity = CGVector(dx: CGFloat.random(in: -30 ... 30), dy: CGFloat.random(in: -30 ... 30));
            
            UIView.animate(withDuration: 5.0,
                           delay: 0.0,
                           options: .curveLinear,
                           animations: { () -> Void in
                            //imageView.transform = CGAffineTransform(rotationAngle: CGFloat((Float(arc4random()) / Float(UINT32_MAX)) * 6.0));
                            //imageView.frame = imageView.frame.offsetBy(dx: velocity.dx, dy: velocity.dy);
                            imageView.transform = CGAffineTransform.identity.translatedBy(x: velocity.dx, y: velocity.dy).rotated(by: CGFloat.random(in: -6.14 ... 6.14)).scaledBy(x: 1, y: 1);
                            imageView.alpha = 1;
            }, completion: { (finished) -> Void in
                UIView.animate(withDuration: 0.2,
                               delay: 0.0,
                               options: .curveEaseOut,
                               animations: { () -> Void in
                                imageView.alpha = 0;
                }, completion: { (finished) -> Void in
                    imageView.removeFromSuperview();
                    self.circles.remove(at: self.circles.firstIndex(of: imageView)!);
                })
            })
            
            Timer.scheduledTimer(timeInterval: Double.random(in: 4.0 ... 7.0), target: self, selector: #selector(self.addCircle), userInfo: nil, repeats: false)
        } else {
            Timer.scheduledTimer(timeInterval: Double.random(in: 0.0 ... 2.0), target: self, selector: #selector(self.addCircle), userInfo: nil, repeats: false)
        }
    }
}

class Circle: UIView {
    var strokeColor: UIColor
    var fillColor: UIColor
    init(frame: CGRect, strokeColor: UIColor, fillColor: UIColor = .clear) {
        self.strokeColor = strokeColor
        self.fillColor = fillColor
        super.init(frame: frame)
        
        self.backgroundColor = .clear;
        self.clipsToBounds = false;
        
        let displayLink = CADisplayLink(target: self, selector: #selector(step));
        
        displayLink.add(to: .current,
                        forMode: .default)
    }
    
    @objc func step(displaylink: CADisplayLink) {
        self.setNeedsDisplay();
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func draw(_ rect: CGRect) {
        if let player = self.layer.presentation() {
            let radius = (frame.height / 2) - (25 * (frame.height / player.frame.height));
            let circlePath = UIBezierPath(arcCenter: CGPoint(x: frame.width / 2, y: frame.height / 2), radius: radius, startAngle: CGFloat(0), endAngle: CGFloat.pi * 2, clockwise: true)
            let x = (player.frame.height - 100) / (frame.height - 180);
            let a = 2 * x;
            let b = a - 1;
            let c = b * b;
            let alph = 0.2 * (1 - (c));
            strokeColor.withAlphaComponent(alph).setStroke();
            fillColor.setFill()
            circlePath.lineWidth = 50 * (frame.height / player.frame.height);
            circlePath.stroke()
        }
    }
}
