//
//  SplashViewController.swift
//  swoof
//
//  Created by Dillon DuPont on 12/10/18.
//  Copyright © 2018 Dillon DuPont. All rights reserved.
//

import UIKit
import SnapKit;

class SplashViewController: UIViewController {

    @IBOutlet weak var ringsView: RingsView!
    @IBOutlet weak var swoofLabel: UILabel!
    @IBOutlet weak var splashLabel: UILabel!
    
    @IBOutlet weak var buttonBase: GradientButtonView!
    @IBOutlet weak var buttonView: GradientView!
    @IBOutlet weak var buttonLabel: UILabel!
    
    @IBOutlet weak var privacyPolicy: UILabel!
    
    override var preferredStatusBarStyle : UIStatusBarStyle {
        return .lightContent
    }
    
    var displayedHelp = false;
    var helpPage = 1;
    
    var vc : HelpPageViewController? = nil;
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.setNeedsStatusBarAppearanceUpdate()
        
        self.buttonBase.onPress = {
            if(!self.displayedHelp) {
                self.displayedHelp = true;
                self.showHelp();
            } else {
                self.helpPage = self.helpPage + 1;
                self.vc!.setViewControllers(
                    [self.storyboard!.instantiateViewController(withIdentifier: "Help\(self.helpPage)")], direction: .forward, animated: true, completion: nil)
                
                if(self.helpPage == 3) {
                    let spring = UISpringTimingParameters(dampingRatio: 0.5, frequencyResponse: 0.4);
                    
                    let animator = UIViewPropertyAnimator(duration: 0, timingParameters: spring);
                    
                    animator.addAnimations {
                        let width = self.view.frame.width - 20;
                        self.buttonBase.frame = CGRect(x: self.buttonBase.center.x - (width / 2), y: self.buttonBase.center.y - 32, width: width, height: 64);
                        self.buttonView.frame = CGRect(x: 0, y: -5, width: width, height: 64);
                        self.buttonLabel.frame = CGRect(x: 0, y: 0, width: width, height: 64);
                        self.buttonLabel.text = "enough readin, gimme a demo";
                        
                        self.buttonBase.shadowColor = .clear;
                        
                        self.buttonBase.snp.updateConstraints { (make) -> Void in
                            make.width.equalTo(width);
                        }
                    }
                    
                    animator.startAnimation();
                }
            }
        }
        
        swoofLabel.snp.makeConstraints { (make) -> Void in
            make.center.equalToSuperview();
        }
        splashLabel.snp.makeConstraints { (make) -> Void in
            make.centerX.equalToSuperview();
            make.top.equalTo(swoofLabel.snp.bottom).offset(10);
        }
        privacyPolicy.snp.makeConstraints { (make) -> Void in
            make.centerX.equalToSuperview();
            make.bottom.equalToSuperview().offset(-20);
        }
        buttonBase.snp.makeConstraints { (make) -> Void in
            make.centerX.equalToSuperview();
            make.width.equalTo(240);
            make.height.equalTo(64);
            make.bottom.equalToSuperview().offset(-50);
        }
        ringsView.snp.makeConstraints { (make) -> Void in
            make.size.equalToSuperview();
            make.center.equalTo(swoofLabel.snp.center);
        }
        ringsView.layoutIfNeeded();
        
        ringsView.setup();
    }
    
    func showHelp() {
        let spring = UISpringTimingParameters(dampingRatio: 0.5, frequencyResponse: 0.4);
        
        let animator = UIViewPropertyAnimator(duration: 0, timingParameters: spring);
        
        animator.addAnimations {
            self.buttonBase.frame = CGRect(x: self.buttonBase.center.x - 32, y: self.buttonBase.center.y - 32, width: 64, height: 64);
            self.buttonView.frame = CGRect(x: 0, y: -5, width: 64, height: 64);
            self.buttonLabel.frame = CGRect(x: 0, y: 0, width: 64, height: 64);
            self.buttonLabel.text = "k.";
            
            self.buttonBase.shadowColor = .clear;
            
            self.buttonBase.snp.updateConstraints { (make) -> Void in
                make.width.equalTo(64);
            }
        }
        
        let newView = UIView(frame: CGRect(x: 0, y: 0, width: 32, height: 32));
        newView.backgroundColor = .white;
        newView.center = self.buttonBase.center;
        newView.layer.cornerRadius = 16;
        
        self.view.addSubview(newView);
        self.view.bringSubviewToFront(self.buttonBase);
        
        let animator2 = UIViewPropertyAnimator(duration: 0, timingParameters: spring);
        
        animator2.addAnimations {
            let radius = max(self.view.frame.height, self.view.frame.width) * 2.0;
            newView.frame = CGRect(x: 0, y: 0, width: radius, height: radius);
            newView.center = self.buttonBase.center;
            newView.layer.cornerRadius = radius / 2.0;
        }
        
        animator2.startAnimation(afterDelay: 0.3)
        animator.startAnimation()
        
        self.vc = (self.storyboard?.instantiateViewController(withIdentifier: "Help") as! HelpPageViewController);
        self.present(self.vc!, animated: true, completion: {
            self.vc!.view.superview!.isUserInteractionEnabled = false;
        });
        
        self.vc!.setViewControllers(
            [self.storyboard!.instantiateViewController(withIdentifier: "Help\(helpPage)")], direction: .forward, animated: false, completion: nil)
    }

    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destination.
        // Pass the selected object to the new view controller.
    }
    */

}
